#!/usr/bin/env python
# Programmer : zhuxp
# Date: 
# Last-modified: 05-28-2014, 14:07:51 EDT
from __future__ import print_function
VERSION="0.1"
import os,sys,argparse
from bam2x.Annotation import BED12
from bam2x import TableIO,Tools
from bam2x import IO
import time
import multiprocessing as mp
def Main():
    print(rep([0,0,1,1,2,2]))

def rep(list):
    s=""
    last=list[0]
    step=0
    offset=0
    for i in list:
        if i!=last:
            s+="{offset},{value},{step}\n".format(offset=offset,value=last,step=step)
            last=i
            offset+=step
            step=1
        else:
            step+=1
    s+="{offset},{value},{step}".format(offset=offset,value=last,step=step)
    return s



    
if __name__=="__main__":
    Main()








from __future__ import print_function
import os
import sys
import logging
import argparse
from bam2x import TableIO,IO,DBI
from bam2x.Tools import seq_wrapper,rc
from random import random
def help():
    return "generate random fastq file"
def set_parser(parser):
    #parser.add_argument("-m",type=str,choices=("seq","cDNA","cdna","cds","utr5","utr3"),dest="method",default="seq")
    parser.add_argument("-g","--genome",type=str,dest="genome",help="chromsome.2bit file")
    parser.add_argument("-b","--bed_column_number",type=int,choices=[3,6,12],dest="bed_column_number",default=12) 
    parser.add_argument("-l","--length",type=int,dest="length",default=60) 
    parser.add_argument("--num",type=int,dest="number",default=100000) 
    
def run(args):
    bedformat="bed"+str(args.bed_column_number)
    dbi=DBI.init(args.genome,"genome")
    out1=IO.fopen(args.output+"_1.fq","w")
    out2=IO.fopen(args.output+"_2.fq","w")
    seqs=[dbi.query(i,method="cdna") for i in TableIO.parse(IO.fopen(args.input,"r"),bedformat)]
    size=len(seqs)
    i=0
    while i<args.number:
        k=int(random()*size)
        l=len(seqs[k])
        if l<args.length: continue
        start=int(random()*(l-args.length))
        length=int(args.length+(l-args.length)*random())
        stop=start+length
        if stop > l: continue
        a=seqs[k][start:start+args.length]
        b=seqs[k][stop-args.length:stop]
        b=rc(b)
        print("@r{i}_1".format(i=i),file=out1)
        print("{a}".format(a=a),file=out1)
        print("+",file=out1)
        print("".join(["I" for j in xrange(args.length)]),file=out1)
        print("@r{i}_2".format(i=i),file=out2)
        print("{b}".format(b=b),file=out2)
        print("+",file=out2)
        print("".join(["I" for j in xrange(args.length)]),file=out2)
        i+=1
        if (i%100==0) :
            print("generate {i} fq".format(i=i),end="\r",file=sys.stderr)



if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())

 



from __future__ import print_function
import os
import sys
import logging
import argparse
from  bam2x import IO,TableIO,DBI
import itertools
from bam2x.Tools import compatible,extend_slice,compatible_with_transcript,_translate,overlap
import logging
'''
Step 1. Group Bed12 into Gene Models(Cluster Isoforms)
Step 2. Compare reads with Gene Models.
'''
def help():
    return "compare bam with bed12. input is bed12 file. aggregation plot nearby tss, nearby tts , and gene body"
def set_parser(parser):
    pass
    #parser.add_argument("-m",type=str,choices=("seq","cDNA","cdna","cds","utr5","utr3"),dest="method")
    parser.add_argument("-b",type=str,dest="bam",help="bam file");
    parser.add_argument("-s","--strand",type=str,dest="strand",choices=("read1","read2"),default="read1",help="strand default:%(default)s");
    parser.add_argument("--bp",type=int,dest="bp",default=2000,help="upstream/downstream bp default:%(default)s");
    #parser.add_argument("-R","--RNASeq",action='store_true',dest="RNASeq",help="if it is RNASeq");
def getNM(bed):
    return int(bed.itemRgb.split(",")[0])
def run(args):
    logging.basicConfig(level=logging.DEBUG) 
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    bam=DBI.init(args.bam,"bam");
    beds=[i for i in TableIO.parse(fin,"bed12")]
    beds.sort()
    bp=args.bp
    print("mapped:{}".format(bam.mapped))
    print("unmapped:{}".format(bam.unmapped))
    data={}
    for i,x in enumerate(iter_cluster(beds)):
        print("{}\t{}:{}-{}".format(i+1,x["chr"],x["start"]+1,x["stop"]))
        '''
        cds=[z.cds() for z in x["beds"] if z.cds()]
        utr3=[z.utr3() for z in x["beds"] if z.utr3()]
        utr5=[z.utr5() for z in x["beds"] if z.utr5()]
        '''
        
        coords = [ up_down_coordinate(gene,args.bp,args.bp) for gene in x["beds"] ]
        for j,y in enumerate(coords):
            data[y.id]={}
            data[y.id]["coord"]=y
            data[y.id]["values"]=[0.0 for l in range(y.cdna_length())];
        coord_beds = [ _translate(coord,bed) for coord,bed in itertools.izip(coords,x["beds"])]
        for j,read in enumerate(bam.query(method="bam1",chr=x["chr"],start=x["start"]-args.bp,stop=x["stop"]+args.bp,strand=args.strand)):
            NM=getNM(read)  # number of hits
            NC=0            # number of compatible 
            c_coords=[]
            for k,coord in enumerate(coords):
                if overlap(read,coord) and compatible(read,coord): # don't consider the reads extend out of coords.
                    NC+=1
                    c_coords.append(k)
            for k,c in enumerate(c_coords):
                coord=coords[c]
                if read.start < coord.start or read.stop > coord.stop:
                    start=max(read.start,coord.start)
                    stop=min(read.stop,coord.stop)
                    read=read._slice(start,stop)
                read_in_coord = _translate(coord,read)
                for l in xrange(read_in_coord.start,read_in_coord.stop):
                    data[coord.id]["values"][l]+=1.0/NC/NM
        for j,y in enumerate(coords):
            print(data[y.id]["coord"])
            print(data[y.id]["values"])

    '''
    for i in data.keys():
        print(data[i]["coord"])
        print(data[i]["values"])
    ''' 


                    
            




def up_down_coordinate(gene,up=1000,down=1000,chromSizes={}):
    new_start=gene.start-up
    if new_start<0: 
        new_start=0
        if gene.strand=="+" or gene.strand==".":
            up=start
        elif gene.strand=="-":
            down=start
    new_stop=gene.stop+down
    if chromSizes.has_key(gene.chr):
        chromSize=chromSizes[gene.chr]
        if new_stop>chromSize:
            new_stop=chromSize
            if strand=="+" or strand==".":
                down=new_stop-gene.stop
            elif strand=="-":
                up=new_stop-gene.stop
    coord=extend_slice(gene,new_start,new_stop)
    coord=coord._replace(id=coord.id+"_up"+str(up)+"_down"+str(down))
    return coord


def iter_cluster(beds):
    retv={}
    cluster=[]
    last=beds[0]
    retv["start"]=last.start
    retv["chr"]=last.chr
    for i in beds:
        if i.chr==last.chr and i.start < last.stop:
            cluster.append(i)
            if i.stop > last.stop:
                last = i

        else:
            if len(cluster) > 0 :
                retv["stop"]=last.stop
                retv["beds"]=cluster
                yield retv
            cluster=[i]
            last=i
            retv={}
            retv["start"]=last.start
            retv["chr"]=last.chr
    if len(cluster) > 0 : 
        retv["stop"]=last.stop
        retv["beds"]=cluster
        yield retv 


if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())





from __future__ import print_function
import os
import sys
import logging
import argparse
from bam2x import TableIO,IO,DBI
from bam2x.Tools import seq_wrapper
from bam2x.TableIO import hclass
def help():
    return "query tabix file"
def set_parser(parser):
    parser.add_argument("-t","--tabix",type=str,dest="db",help="tabix db file")
    parser.add_argument("-A","--annotation_type",type=str,dest="type",choices=hclass.keys(),default=None,help="tabix entry type , default is None")
    parser.add_argument("-I","--format",type=str,choices=hclass.keys(),dest="format",default="bed12",help="input format : %(default)s") 
    
def run(args):
    out=IO.fopen(args.output,"w")
    cls=None
    if hclass.has_key(args.type):
        cls=hclass[args.type]
        dbi=DBI.init(args.db,"tabix",cls=cls)
    else:
        dbi=DBI.init(args.db,"tabix")
    for i in TableIO.parse(IO.fopen(args.input,"r"),args.format):
        print("QR",i,file=out)
        
        for j,ht in enumerate(dbi.query(i)):
            print("HT_{k}\t{ht}".format(k=j+1,ht=ht),file=out)

if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())

 



from __future__ import print_function
import os
import sys
import logging
import argparse
from  bam2x import IO,TableIO,DBI
from bam2x.Tools import compatible
def help():
    return "group bed12 into clusters and groups."
def set_parser(parser):
    pass
    #parser.add_argument("-m",type=str,choices=("seq","cDNA","cdna","cds","utr5","utr3"),dest="method")
    
    
def run(args):
    fin=IO.fopen(args.input,"r")
    out=IO.fopen(args.output,"w")
    beds=[i for i in TableIO.parse(fin,"bed12")]
    beds.sort()
    for i,x in enumerate(iter_cluster(beds)):
        print("CL_"+str(i+1),file=out)
        for j,y in enumerate(greedy_iter_compatible_group(x)):
            print("\tGROUP{j}".format(j=j+1),file=out)
            for k,z in enumerate(sorted(y,key= lambda x0:x0.cdna_length(), reverse=True)):
                print("\tCL.{i}_GP.{j}_TR.{k}\t{l}\t{z}".format(i=i+1,j=j+1,k=k+1,l=z.cdna_length(),z=z),file=out)


def greedy_iter_compatible_group(beds):
    local_beds=[i for i in beds]
    compatible_groups=[[]]
    for i in local_beds:
        sign=False
        for j,c in enumerate(compatible_groups):
            if compatible_with_group(i,c):
                c.append(i)
                sign=True
                break
        if not sign:
            compatible_groups.append([i])

    for i in sorted(compatible_groups):
        yield i








def compatible_with_group(i,group):
    for j in group:
        if not compatible(i,j):
            return False
    return True




def iter_cluster(beds):
    cluster=[]
    last=beds[0]
    for i in beds:
        if i.chr==last.chr and i.start < last.stop:
            cluster.append(i)
            if i.stop > last.stop:
                last = i
        else:
            if len(cluster) > 0 : yield cluster
            cluster=[i]
            last=i
    if len(cluster) > 0 : yield cluster


if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())










from __future__ import print_function
import os
import sys
import logging
import argparse
from bam2x import TableIO,IO,DBI
from bam2x.Tools import seq_wrapper
def help():
    return "query bigwig file"
def set_parser(parser):
    parser.add_argument("-m",type=str,choices=("DNA","cDNA"),dest="method",default="DNA",help="method default: %(default)s")
    parser.add_argument("-b","--bigwig",type=str,dest="bw",help="bigwig file")
    parser.add_argument("-I","--format",type=str,choices=("bed3","bed6","bed12","vcf"),dest="format",default="bed12",help="input format : %(defualt)s") 
    
def run(args):
    dbi=DBI.init(args.bw,"bigwig")
    out=IO.fopen(args.output,"w")
    for i in TableIO.parse(IO.fopen(args.input,"r"),args.format):
        ht=[ j for j in dbi.query(i,method=args.method) ]
        print("QR",i,file=out)
        print("HT",ht,file=out)

if __name__=="__main__":
    from bam2x.IO import parser_factory
    p=parser_factory(description=help())
    set_parser(p)
    if len(sys.argv)==1:
        print(p.print_help())
        exit(0)
    run(p.parse_args())

 
